<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Schema;

class CreateProjectsTable extends Migration
{
    /**
     * Run the migrations.
     *
     * @return void
     */
    public function up()
    {
        Schema::create('projects', function (Blueprint $table) {
            $table->bigIncrements('id');
            
            $table->boolean('verified')->default(0);

            $table->unsignedBigInteger('proficiency_id');
            $table->foreign('proficiency_id')->references('id')->on('proficiencies')->onDelete('cascade');

            $table->unsignedBigInteger('client_id');
            $table->foreign('client_id')->references('id')->on('clients')->onDelete('cascade');

            $table->unsignedBigInteger('technician_id')->nullable();
            $table->foreign('technician_id')->references('id')->on('technicians')->onDelete('cascade');

            $table->unsignedBigInteger('city_id')->nullable();
            $table->foreign('city_id')->references('id')->on('cities')->onDelete('cascade');

            $table->unsignedBigInteger('address_id');
            $table->foreign('address_id')->references('id')->on('addresses')->onDelete('cascade');

            $table->dateTime('start_at');
            $table->decimal('budget', $precision = 10, $scale = 2);
            $table->decimal('approximate_cost', $precision = 10, $scale = 2)->nullable();
            $table->text('details');
            $table->boolean('vip')->default(0);
            $table->enum('client_status', ['cancel', 'current', 'accepted', 'done']);
            $table->enum('technician_status', ['cancel', 'todo', 'done'])->nullable();
            $table->integer('discount')->nullable();

            $table->softDeletes();
            $table->timestamps();
        });
    }

    /**
     * Reverse the migrations.
     *
     * @return void
     */
    public function down()
    {
        Schema::dropIfExists('projects');
    }
}
