<?php

use App\DB\AdminRepo;
use App\Http\Controllers\AddressController;
use App\Http\Controllers\AdminController;
use App\Http\Controllers\ChatController;
use App\Http\Controllers\ClientController;
use App\Http\Controllers\ProjectController;
use App\Http\Controllers\OtpController;
use App\Http\Controllers\TechnicianController;
use App\Http\Controllers\HomeController;
use App\Models\Project;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Route;
use phpDocumentor\Reflection\ProjectFactory;

/*
|--------------------------------------------------------------------------
| API Routes
|--------------------------------------------------------------------------
|
| Here is where you can register API routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| is assigned the "api" middleware group. Enjoy building your API!
|
*/

Auth::routes();

Route::middleware('auth:api')->group(function () {

    Route::get('/is-auth', [HomeController::class, 'isAuth']);

    Route::get('/logout', [OtpController::class, 'logout']);

    // notifications api
    // show all
    Route::get('/notifications', [HomeController::class, 'getNotifications']);
    // read
    Route::patch('/notifications', [HomeController::class, 'seenNotification']);
    // has new notification
    Route::get('/new-notification', [HomeController::class, 'hasUnreadNotification']);


    // messages
    Route::get('/messages', [ChatController::class, 'index']);
    Route::post('/send-message', [ChatController::class, 'sendMessage']);
    Route::patch('/read-message', [ChatController::class, 'read']);
    Route::get('/conversation', [ChatController::class, 'fetchConversation']);
    Route::post('/start-conversation', [ChatController::class, 'startConversation']);
    Route::get('/goto-conversation', [ChatController::class, 'gotoConversation']);
    Route::get('/goto-conversation-technician', [ChatController::class, 'gotoConversationTechnician']);

    // save project
    Route::post('/projects', [ProjectController::class, 'submitProject']);
    //upload photos of project
    Route::post('/upload-photo', [ProjectController::class, 'uploadPhoto']);
    // delete photo of project
    Route::post('/delete-photo', [ProjectController::class, 'deletePhoto']);
    //check discount code validation
    Route::get('/check-discount/{code}', [ProjectController::class, 'checkDiscountCode']);

    // address
    Route::get('/addresses', [AddressController::class, 'fetchAddresses']);
    Route::post('/addresses', [AddressController::class, 'newAddress']);

    // client panel api list
    // k^DXBP0cp~Ef
    // account
    Route::get('/client/edit-account', [ClientController::class, 'fetch']);
    Route::patch('/client/edit-account', [ClientController::class, 'account']);
    Route::post('/client-avatar', [ClientController::class, 'changeProfilePicture']);
    Route::delete('/addresses/{id}', [AddressController::class, 'destroy']);

    // my orders
    Route::get('/current-orders', [ProjectController::class, 'fetchCurrentOrders']);
    // current project page api
    Route::get('/current-orders/{id}', [ProjectController::class, 'fetchCurrentOrder']);

    Route::patch('/accept-bid', [ProjectController::class, 'acceptBid']);

    // Route::get('/accepted-orders', [ProjectController::class, 'fetchAcceptedOrders']);
    // accepted project page api
    Route::get('/accepted-orders/{id}', [ProjectController::class, 'fetchAcceptedOrder']);

    Route::get('/done-orders', [ProjectController::class, 'fetchDoneOrders']);
    //  done project page api
    Route::get('/done-orders/{id}', [ProjectController::class, 'fetchDoneOrder']);

    Route::get('/canceled-orders', [ProjectController::class, 'fetchCanceledOrders']);
    // canceled project page api
    Route::get('canceled-orders/{id}', [ProjectController::class, 'fetchCanceledOrder']);

    // cancel a project(client)
    Route::patch('/cancel-order', [ProjectController::class, 'cancelOrder']);

    // balance and transactions
    Route::get('/transactions', [ClientController::class, 'fetchTransactions']);
    Route::get('/balance', [ClientController::class, 'balance']);

    Route::post('/withdraw-request', [ClientController::class, 'withdraw']);

    // payments
    Route::post('/deposit', [ClientController::class, 'deposit']);
    Route::post('/verify-deposit', [ClientController::class, 'verifyDeposit']);

    Route::post('/credit-payment', [ProjectController::class, 'creditPayment']);
    Route::post('/cash-payment', [ProjectController::class, 'cashPayment']);

    //  rate and comment for done project from client to technician
    Route::post('/rate-comment/client', [ProjectController::class, 'rateAndCommentToTechnician']);

    // saved technician
    Route::get('/saved-technicians', [ClientController::class, 'fetchSavedTechnicians']);
    // unsave:
    Route::delete('/saved-technicians/{id}', [ClientController::class, 'unsaveTechnician']);
    Route::post('/saved-technicians/{id}', [ClientController::class, 'saveTechnician']);

    // invite friends
    Route::get('/invite-code', [ClientController::class, 'inviteCode']);

    // technician profile show to client
    Route::get('/technician-profile/{id}', [ClientController::class, 'fetchTechnician']);

    // technician panel api list

    // dashboard
    Route::get('/weekly-income', [TechnicianController::class, 'weekly']);
    Route::get('/monthly-income', [TechnicianController::class, 'monthly']);
    Route::get('/yearly-income', [TechnicianController::class, 'yearly']);
    Route::get('/total-income', [TechnicianController::class, 'fetchTotalIncome']);

    // account
    Route::get('technician/edit-account', [TechnicianController::class, 'fetch']);
    Route::post('technician/edit-account', [TechnicianController::class, 'account']);
    Route::post('/technician-avatar', [TechnicianController::class, 'changeProfilePicture']);
    Route::post('/national-card', [TechnicianController::class, 'uploadNationalCard']);

    // history

    Route::get('/todo-projects', [ProjectController::class, 'fetchTodoProjects']);
    Route::get('/done-projects', [ProjectController::class, 'fetchDoneProjects']);

    // todo project page api
    Route::get('/todo-projects/{id}', [ProjectController::class, 'fetchTodoProject']);
    Route::get('/done-projects/{id}', [ProjectController::class, 'fetchDoneProject']);

    // set photos uploaded from technician to the project
    Route::post('/submit-photos', [ProjectController::class, 'submitPhotos']);

    // cancel a project(technician)
    Route::patch('/cancel-project', [ProjectController::class, 'cancelProject']);

    // profile
    Route::get('/resume', [TechnicianController::class, 'resume']);

    // projects page api
    Route::get('/projects', [ProjectController::class, 'fetchProjects']);
    Route::get('/towns', [AddressController::class, 'allTowns']);
    Route::get('/projects/{id}', [ProjectController::class, 'fetchProject']);

    // client profile show to technician
    Route::get('/client-profile/{id}', [TechnicianController::class, 'fetchClient']);

    // technician comment for current project
    Route::post('/add-comment', [TechnicianController::class, 'commentToProject']);

    //  rate and comment for done project from technician to client
    Route::post('/rate-comment/technician', [ProjectController::class, 'rateAndCommentToClient']);

    // send a bid from technician to client
    Route::post('/make-bid', [TechnicianController::class, 'makeBid']);
});

// admin panel api list
Route::group(['middleware' => ['auth:api', 'checkAdmin']], function () {

    // dashboard charts
    Route::get('/dashboard-reports', [AdminController::class, 'fetchDashboardReports']);

    // technicians management
    Route::get('/notverified-technicians', [AdminController::class, 'fetchNotVerifiedTechnicians']);
    Route::get('/certificates', [AdminController::class, 'fetchCertificates']);
    Route::post('/certificates', [AdminController::class, 'addCertificate']);
    Route::delete('/certificates/{id}', [AdminController::class, 'deleteCertificate']);
    Route::get('/verified-technicians', [AdminController::class, 'fetchVerifiedTechnicians']);
    Route::get('/all-technicians', [AdminController::class, 'fetchAllTechnicians']);
    Route::get('/technician/{id}', [AdminController::class, 'fetchTechnician']);
    Route::post('/verify-technician', [AdminController::class, 'verifyTechnician']);
    Route::post('/set-vip', [AdminController::class, 'setTechnicianVip']);
    Route::get('/incomes', [AdminController::class, 'fetchIncomes']);
    Route::get('/incomes/{id}', [AdminController::class, 'fetchIncomesDetail']);

    // clients management
    Route::get('/all-clients2', [AdminController::class, 'fetchAllClients']);
    Route::get('/client/{id}', [AdminController::class, 'fetchClient']);
    Route::get('/clients-balance', [AdminController::class, 'fetchBalanceClient']);
    Route::get('/transactions/{id}', [AdminController::class, 'fetchTransactionsClient']);
    Route::get('/withdraw-requests', [AdminController::class, 'fetchWithdrawRequests']);
    Route::get('/withdraw/{id}', [AdminController::class, 'fetchWithdrawRequest']);
    Route::put('/withdraw-status', [AdminController::class, 'setWithdrawStatus']);
    Route::patch('/paid', [AdminController::class, 'paidTechnicianIncome']);

    // projects management
    Route::get('/projects/{type}/{status}', [AdminController::class, 'fetchProjects']);
    Route::post('/verify-project', [AdminController::class, 'verifyProject']);
    Route::get('/project/{id}', [AdminController::class, 'fetchProject']);
    Route::get('/vip-technicians/{proficiency_id}', [AdminController::class, 'fetchVipTechnicians']);
    Route::delete('/bids/{id}', [AdminController::class, 'deleteBid']);
    Route::patch('/submit-bid', [AdminController::class, 'submitBid']);

    // comments management
    Route::get('/comments/project', [AdminController::class, 'fetchCommentsOnProject']);
    Route::get('/comments/technician', [AdminController::class, 'fetchCommentsOnTechnician']);
    Route::get('/comments/client', [AdminController::class, 'fetchCommentsOnClient']);
    Route::delete('/comments/{id}', [AdminController::class, 'deleteComment']);

    // messages management
    Route::get('/messages/technicians', [AdminController::class, 'fetchTechniciansMessages']);
    Route::get('/messages/clients', [AdminController::class, 'fetchClientsMessages']);

    // cities management
    Route::post('/cities', [AdminController::class, 'createCity']);
    Route::delete('/cities/{id}', [AdminController::class, 'deleteCity']);

    // towns management
    Route::get('/all-towns', [AdminController::class, 'fetchTowns']);
    Route::post('/towns', [AdminController::class, 'createTown']);
    Route::delete('/towns/{id}', [AdminController::class, 'deleteTown']);

    // skills management
    Route::get('/skills', [AdminController::class, 'fetchSkills']);
    Route::post('/skills', [AdminController::class, 'createSkill']);
    Route::delete('/skills/{id}', [AdminController::class, 'deleteSkill']);

    // products management
    Route::get('/all-products', [AdminController::class, 'fetchProducts']);
    Route::post('/products', [AdminController::class, 'createProduct']);
    Route::delete('/products/{id}', [AdminController::class, 'deleteProduct']);

    // discount management
    Route::get('/discounts', [AdminController::class, 'fetchDiscounts']);
    Route::post('/discounts', [AdminController::class, 'createDiscount']);
    Route::delete('/discounts/{id}', [AdminController::class, 'deleteDiscount']);

    // send notifiacion to users
    Route::post('/send-notification', [AdminController::class, 'sendNotification']);
    Route::get('/all-notifications', [AdminController::class, 'fetchNotifications']);

    Route::get('/all-users', [AdminController::class, 'fetchUsers']);

    // settings in panel
    Route::post('/settings', [AdminController::class, 'submitSettings']);
});


Route::group(['middleware' => ['cors', 'json.response']], function () {
    // otp api list
    Route::post('/send-otp', [OtpController::class, 'sendOtp']);
    Route::post('/verify-otp', [OtpController::class, 'checkOtp']);
    Route::post('/sign-up', [OtpController::class, 'register']);

    //admin panel login api
    Route::post('/login', [AdminController::class, 'login']);
    // Route::post('/register', [AdminController::class, 'register']);
});

// fetch settings
Route::get('/settings', [HomeController::class, 'fetchSettings']);


// home page api list
Route::get('/mobile-home', [HomeController::class, 'mobileHome']);
Route::get('/search');
Route::get('/popular-categories', [HomeController::class, 'fetchPopularCats']);
Route::get('/last-orders', [HomeController::class, 'fetchLastOrders']);
Route::get('/vip-technicians', [HomeController::class, 'fetchVipTechnicians']);

Route::get('/cities', [AddressController::class, 'fetchCities']);
Route::get('/cities/{id}/towns', [AddressController::class, 'fetchTowns']);

// categories page api list
Route::get('/search-skills', [ProjectController::class, 'searchOnCategories']);

// tasks page for client
Route::get('/tasks', [ProjectController::class, 'fetchTasks']);

// new project api:

Route::get('/proficiencies', [ProjectController::class, 'fetchProficiencies']);
Route::get('/proficiencies/{id}/skills', [ProjectController::class, 'fetchSkills']);

// fetch product of skill
Route::get('/products', [ProjectController::class, 'fetchProducts']);

// apply discount code
Route::post('/register-discount');


// the project finish? true/false
Route::get('/finished-project');
